CHECK_SETTING = "Check Settings keyword"
UFG_RELATED = "Ultrafast Grid only"
TARGET_SUPPORT = "Allow to pass Target keyword"
CHECK_SETTINGS_SUPPORT = "Allow to pass Check Settings keywords"
CHECK_FLOW = "Check flow"
TARGET_KEYWORD = "Target Keyword"
